DEFAULT_SETTINGS = {
    "truncated_normal": {
        "hit_ratio_threshold": 0.3,
        "expand_ratio": 1.5,
    },
    "truncated_multivariate_normal": {
        "hit_ratio_threshold": 0.3,
        "expand_ratio": 1.5,
    }
}
